***********************************************************
*             DISK FORMAT FOR TINY GIANT                  *
*  COPYRIGHT 1987 BY JOSEPH BARTEL - ALL RIGHTS RESERVED  *
***********************************************************
*
***********************************************************
*			TRACK PATTERN			  *
*							  *
*	COUNT 	VALUE (HEX)	NOTES			  *
*							  *
*	50	4E		GAP I  (POST-INDEX GAP)	  *
*	12	00					  *
*	3	F5		(WRITES $A1)		  *
*	1	FE		ID ADDRESS MARK		  *
*	1	00..27		TRACK NUMBER		  *
*	1	00..01		SIDE NUMBER		  *
*	1	01..09		SECTOR NUMBER		  *
*	1	02		SECTOR LENGTH (512 BYTES) *
*	1	F7		(WRITES 2 CRC BYTES)	  *
*	22	4E		GAP II  (ID GAP)	  *
*	12	00					  *
*	3	F5		(WRITES $A1)		  *
*	1	FB		DATA ADDRESS MARK	  *
*	512	00..FF		DATA			  *
*	1	F7		(WRITES 2 CRC BYTES)	  *
*	80	4E		GAP III  (DATA GAP)	  *
*	1500	4E		GAP IV  (PRE-INDEX GAP)	  *
*							  *
***********************************************************

STATREG	EQU	$E00001		;1770 STATUS REG
COMMREG	EQU	$E00001		;1770 COMMAND REG
TRACREG	EQU	$E00003		;1770 TRACK REGISTER
SECTREG	EQU	$E00005		;1770 SECTOR REG
DATAREG	EQU	$E00007		;1770 DATA REG
IMRREG	EQU	$E8000B		;68681 INTERRUPT MASK REGISTER
OP0REG	EQU	$E8001D		;68681 OUTPUT BIT SET
OP1REG	EQU	$E8001F		;68681 OUTPUT BIT RESET
GAP1	EQU	50		;POST-INDEX GAP
GAP3	EQU	80		;DATA GAP
IDTR	EQU	GAP1+16		;FIRST ID FIELD TRACK
IDSI	EQU	GAP1+17		;FIRST ID FIELD SIDE
IDSE	EQU	GAP1+18		;FIRST ID FIELD SECTOR
IDNX	EQU	GAP3+572	;OFFSET TO NEXT ID FIELD
*----------------------------------------------------------
	ORG	0
START	LEA	START-2(PC),A6	;SET BASE REG
	LEA	STACK(A6),A7	;SET SP
;------ SIGN ON AND GET STARTED
	LEA	SIGNON(A6),A0
	BSR	SPRINT
	BSR	GETC		;GET REPLY
	CMPI.B	#13,D0		;IF CR, CONTINUE
	BNE	EXIT		;IF NOT, EXIT
	BSR	CRLF
;------- BUILD TRACK PATTERN IN MEMORY
	LEA	WRITBF(A6),A1	;POINT TO BUF
	MOVE.B	#$4E,D1		;GAP 1
	MOVEQ	#GAP1,D0
	BSR	FILL
	MOVE.W	#9,COUNT(A6)	;9 SECTORS PER TRACK
TRK040	BSR	BLDSEC		;USE SECTOR ROUTINE
	SUBQ.W	#1,COUNT(A6)
	BNE	TRK040
	MOVE.B	#$4E,D1		;GAP 4
	MOVE	#1500,D0
	BSR	FILL
;------ PUT IN SECTOR NUMBERS
	LEA	WRITBF+IDSE(A6),A1	;PNT FIRST SECTOR NUM
	MOVEQ	#1,D0		;START SECTOR 1
TRK060	MOVE.B	D0,(A1)		;PUT IN PATTERN
	ADDA.W	#IDNX,A1	;POINT TO NEXT
	ADDQ.B	#1,D0		;NEXT NUM
	CMPI.B	#9,D0		;SECTORS PR TRACK
	BLE	TRK060
;------ RESTORE TO TRAK 0
	BSR	SIDE0		;SELECT DRIVE A:
	BSR	BUSYWAIT	;WAIT FOR NOT BUSY
	MOVE.B	#3,COMMREG	;SEND RESTORE CMD
	BSR	STATDLY		;DELAY FOR STATUS
	BSR	BUSYWAIT	;WAIT UNTIL DONE
	MOVE.B	STATREG,D0	;GET STATUS
	ANDI	#$10,D0
	BNE	ERROR
	MOVE.W	#40,TRKCNT(A6)	;SET TRACK COUNT = 40
;------ TO FORMAT, WRITE BOTH SIDES, READ BOTH SIDES
FORMCYL	BSR	WRITRAC		;FORMAT SIDE 0 OF THIS CYLINDER
	BNE	ERROR
	BSR	READMUL		;READ ALL 9 SECTORS INTO MEMORY
	BNE	ERROR
	BSR	CHKTRAC		;MAKE SURE THEY ARE OKAY
	BNE	ERROR
	BSR	SIDE1
	BSR	WRITRAC		;FORMAT SIDE 1 OF THIS CYLINDER
	BNE	ERROR
	BSR	READMUL		;READ ALL 9 SECTORS INTO MEMORY
	BNE	ERROR
	BSR	CHKTRAC		;MAKE SURE THEY ARE OKAY
	BNE	ERROR
	SUBQ.W	#1,TRKCNT(A6)
	BEQ.S	FORMEND		;IF ON LAST TRACK, EXIT LOOP
	BSR	SIDE0
	BSR	NEXTCYL		;MOVE TO NEXT CYLINDER
	BRA	FORMCYL
FORMEND	NOP
;------ WRITE FATS AND DIRECTORY SECTORS
	LEA	WRITBF(A6),A1	;CREATE SECOND FAT SECTOR
	MOVEQ	#0,D1
	MOVE	#512,D0
	BSR	FILL
	MOVEQ	#2,D0		;WRITE RECORD 2
	BSR	RAWRITE
	BNE	ERROR
	MOVEQ	#4,D0		;WRITE RECORD 4
	BSR	RAWRITE
	BNE	ERROR
	MOVEQ	#2,D0		;READ RECORD 2
	BSR	RAWREAD
	BNE	ERROR
	BSR	CMPREC		;SEE IF IT MATCHES WHAT WAS WRITTEN
;=====	BNE	ERROR
	MOVEQ	#4,D0		;READ RECORD 4
	BSR	RAWREAD
	BNE	ERROR
	BSR	CMPREC		;SEE IF IT MATCHES WHAT WAS WRITTEN
;=====	BNE	ERROR
	LEA	WRITBF(A6),A1	;CREATE FIRST FAT SECTOR
	LEA	FATPAT(A6),A0
	MOVE.B	(A0)+,(A1)+
	MOVE.B	(A0)+,(A1)+
	MOVE.B	(A0)+,(A1)+
	MOVEQ	#1,D0		;WRITE RECORD 1
	BSR	RAWRITE
	BNE	ERROR
	MOVEQ	#3,D0		;WRITE RECORD 3	
	BSR	RAWRITE
	BNE	ERROR
	MOVEQ	#1,D0		;READ RECORD 1
	BSR	RAWREAD
	BNE	ERROR
	BSR	CMPREC		;SEE IF IT MATCHES WHAT WAS WRITTEN
;=====	BNE	ERROR
	MOVEQ	#3,D0		;READ RECORD 3
	BSR	RAWREAD
	BNE	ERROR
	BSR	CMPREC		;SEE IF IT MATCHES WHAT WAS WRITTEN
;=====	BNE	ERROR
	LEA	WRITBF(A6),A1	;CREATE A DIR SECTOR
	MOVEQ	#15,D2		;16 DIR ENTRIES
	MOVE	#$E5,D1
DIR00	CLR.B	(A1)+		;ONE ZERO
	MOVEQ	#31,D0		;THIRTY-ONE $E5
	BSR	FILL
	DBRA	D2,DIR00	;NEXT ENTRY
	MOVEQ	#6,D5		;7 DIR RECORDS
DIR10	MOVEQ	#11,D0
	SUB.L	D5,D0		;RECORD 5 THROUGH 11
	BSR	RAWRITE		;WRITE RECORD
	BNE	ERROR
	MOVEQ	#11,D0
	SUB.L	D5,D0
	BSR	RAWREAD		;READ RECORD
	BNE	ERROR
	BSR	CMPREC		;SEE IF IT MATCHES WHAT WAS WRITTEN
;=====	BNE	ERROR
	DBRA	D5,DIR10	;NEXT RECORD
;------	PUT VOLUME NAME ON DISK
	LEA	VOLMSG(A6),A0	;ASK FOR VOLUME NAME
	BSR	SPRINT
	BSR	GETLN		;GET NAME
	CMPI.W	#13,D0
	BEQ.S	EXIT		;IF NO NAME, EXIT
	LEA	GETBF(A6),A1
	LEA	PARAM(A6),A0
	MOVE.W	8(A0),D0	;GET STRING LENGTH
	CLR.B	-1(A1,D0.W)	;REPLACE CR WITH NULL
	MOVEA.L	A1,A0		;REMOVE LEADING SPACES
	SUBQ.L	#1,A0
SKIPSP	ADDQ.L	#1,A0
	CMPI.B	#$20,(A0)
	BEQ	SKIPSP
REMOVSP	MOVE.B	(A0)+,(A1)+
	BNE	REMOVSP
	LEA	GETBF(A6),A1	;CHECK FOR DRIVE SPEC
	CMPI.B	#':',1(A1)
	BEQ.S	DRIVA
	MOVEM.L	(A1),#$000F	;IF NO SPEC, MAKE ROOM FOR ONE
	CLR.L	D3		;  BUT BE SURE TO TERMINATE NEW STRING
	MOVEM.L	#$000F,2(A1)
	MOVE.L	D1,6(A1)
DRIVA	MOVE.W	#'A:',(A1)	;DRIVE SPEC IS A:
	BSR	CREATE		;CREATE FILENAME
	BNE	ERROR
	BSR	VATTRIB		;CHANGE ATTRIBUTE TO VOLUME NAME
	BNE	ERROR
;------- ALL DONE, EXIT TO K-OS
EXIT	LEA	PARAM(A6),A0	;POINT TO PARAM BLK
	MOVE.W	#54,(A0)	;FUNCTION #54
	CLR.W	2(A0)		;STATUS
	CLR.W	4(A0)		;EXIT CODE
	BRA	XTRAP		;GO DO TRAP
ERROR
	LEA	DSKRMS(A6),A0
	BSR	WRITELN
	LEA	PARAM(A6),A0	;EXIT W/TERMINATION CODE <> 0
	MOVE.W	#54,(A0)+
	CLR.W	(A0)+
	MOVE.L	#1,(A0)+
	BRA	XTRAP
************************ CALLABLE ROUTINES **************************
;------	SELECT SIDE 0 OF DISK A
SIDE0	MOVE.B	#$78,OP1REG	;SELECT DRIVE A: SIDE 0
	MOVE.B	#$80,OP0REG
	LEA	WRITBF+IDSI(A6),A1
	MOVEQ	#8,D0		;FOR EACH ID FIELD IN TRACK IMAGE
	MOVEQ	#0,D1
	MOVE.W	#IDNX,D2
S000	MOVE.B	D1,(A1)		;  SIDE = 0
	ADDA.W	D2,A1
	DBRA	D0,S000
	RTS
;------	SELECT SIDE 1 OF DISK A
SIDE1	MOVE.B	#$70,OP1REG	;SELECT DRIVE A: SIDE 1
	MOVE.B	#$88,OP0REG
	LEA	WRITBF+IDSI(A6),A1
	MOVEQ	#8,D0		;FOR EACH ID FIELD IN TRACK IMAGE
	MOVEQ	#1,D1
	MOVE.W	#IDNX,D2
S100	MOVE.B	D1,(A1)		;  SIDE = 1
	ADDA.W	D2,A1
	DBRA	D0,S100
	RTS
;------	SELECT NEXT CYLINDER
NEXTCYL	BSR	STEPIN		;MOVE HEAD IN ONE TCYLK
	LEA	WRITBF+IDTR(A6),A1
	MOVEQ	#8,D0		;FOR EACH ID FIELD IN TRACK IMAGE
	MOVE.W	#IDNX,D2
NC00	ADDQ.B	#1,(A1)		;  TRACK = TRACK + 1
	ADDA.W	D2,A1
	DBRA	D0,NC00
	RTS
;------	SEE IF READ MULTIPLE IS ALL E5  ( STATUS ==> D0 )
CHKTRAC	LEA	READBF(A6),A0
	MOVE.W	#4607,D0	;FOR 9*512 BYTES
	MOVEQ	#$E5,D1
CT00	CMP.B	(A0)+,D1	;  SHOULD BE E5
	DBNE	D0,CT00
	SNE	D0		;IF NOT ALL E5, RETURN BAD STATUS
	RTS
;------ SEE IF RECORD WRITTEN IS RECORD READ  ( STATUS ==> D0 )
CMPREC	LEA	READBF(A6),A0
	LEA	WRITBF(A6),A1
	MOVE.W	#511,D0		;FOR 512 BYTES
CR00	CMPM.B	(A0)+,(A1)+	;  SHOULD BE THE SAME
	DBNE	D0,CR00
	SNE	D0		;IF NOT THE SAME, RETURN BAD STATUS
	RTS
;------ WAIT FOR DISK CONTOLLER TO FINISH COMMAND
BUSYWAIT
	BTST	#0,STATREG	;REPEAT READ STATUS
	BNE	BUSYWAIT	;UNTIL NOT BUSY
	RTS
;-------- DELAY AFTER COMMAND BEFORE STATUS IS VALID
STATDLY	MOVEQ	#20,D0		;32 USEC @8MHZ=256 CLOCKS
STAT10	DBRA	D0,STAT10	;10 CLOCKS PER LOOP
	RTS			;52 CLOCKS OVERHEAD
;------	STEP HEAD TOWARD TRACK 40
STEPIN	BSR	BUSYWAIT
	MOVE.B	#$53,COMMREG	;STEP HEAD IN, NO VERIFY
	BSR	STATDLY
	BSR	BUSYWAIT
	RTS
;------ WRITE TRACK  ( STATUS ==> D0 )
WRITRAC
	LEA	STATREG,A0	;1770 STATUS REG
	LEA	DATAREG,A2	;1770 DATA REG
	LEA	WRITBF(A6),A1	;POINT TO DATA BUFFER
	MOVEQ	#0,D1
	MOVEQ	#1,D2
	MOVE.B	#0,IMRREG	;MASK ALL 68681 INTERRUPTS
	MOVE.B	#$F0,COMMREG
	BSR	STATDLY
WRIT20	BTST.B	D1,(A0)		;WHILE BUSY ( CRITICAL LOOP, BE QUICK!)
	BEQ.S	WRIT30
	BTST.B	D2,(A0)		;IF NOT DRQ
	BEQ.S	WRIT20		;THEN CONTINUE
	MOVE.B	(A1)+,(A2)	;ELSE WRITE DATA
	BRA.S	WRIT20		;END WHILE ( END OF CRITICAL LOOP)
WRIT30	MOVE.B	#$2A,IMRREG	;UN-MASK TIMER & RECEIVER IRQS
	MOVE.B	STATREG,D0
	ANDI	#$5C,D0		;RETURN STATUS
	RTS
;------ READ MULTIPLE  ( STATUS ==> D0 )
READMUL
	LEA	STATREG,A0	;1770 STATUS REG
	LEA	DATAREG,A2	;1770 DATA REG
	LEA	READBF(A6),A1	;BUFFER TO PUT DATA INTO
	MOVEQ	#0,D1
	MOVEQ	#1,D2
	MOVE.B	D2,SECTREG	;RECORDS 1 THROUGH 9
	MOVE.B	#0,IMRREG	;MASK ALL 68681 INTERRUPTS
	MOVE.B	#$90,COMMREG	;START READ
	BSR	STATDLY
READ20	BTST.B	D1,(A0)		;WHILE BUSY ( CRITICAL LOOP)
	BEQ.S	READ30
	BTST.B	D2,(A0)		;IF NOT DRQ
	BEQ.S	READ20		;LOOP
	MOVE.B	(A2),(A1)+	;GRAB BYTE
	BRA.S	READ20		;END WHILE (END OF CRITICAL LOOP )
READ30	MOVE.B	#$2A,IMRREG	;UN-MASK TIMER & RECEIVER IRQS
	MOVE.B	STATREG,D0
	ANDI	#$0C,D0		;RETURN STATUS
	CMPI.B	#10,SECTREG	;DID WE GET THROUGH ALL 9 SECTORS?
	SLO	D0
	RTS
;------	RAW WRITE  ( D0 ==> RECORD,  STATUS ==> D0 )
RAWRITE	LEA	PARAM(A6),A0
	MOVE.W	#23,(A0)+
	CLR.W	(A0)+
	MOVE.W	#$0100,(A0)+
	MOVE.W	D0,(A0)+
	LEA	WRITBF(A6),A1
	MOVE.L	A1,(A0)+
	BSR	XTRAP
	MOVE.B	3(A0),D0	;RETURN STATUS
	RTS
;------	RAW READ  ( D0 ==> RECORD,  STATUS ==> D0 )
RAWREAD	LEA	PARAM(A6),A0
	MOVE.W	#22,(A0)+
	CLR.W	(A0)+
	MOVE.W	#$0100,(A0)+
	MOVE.W	D0,(A0)+
	LEA	READBF(A6),A1
	MOVE.L	A1,(A0)+
	BSR	XTRAP
	MOVE.B	3(A0),D0	;RETURN STATUS
	RTS
;------	CREATE FILE
CREATE	LEA	PARAM(A6),A0
	MOVE.W	#6,(A0)+
	CLR.W	(A0)+
	LEA	GETBF(A6),A1
	MOVE.L	A1,(A0)+
	BSR	XTRAP
	MOVE.B	3(A0),D0	;RETURN STATUS
	RTS
;------	SET FILE ATTRIBUTES TO VOLUME NAME
VATTRIB	LEA	PARAM(A6),A0
	MOVE.W	#19,(A0)+
	CLR.W	(A0)+
	LEA	GETBF(A6),A1
	MOVE.L	A1,(A0)+
	MOVE.L	#8,(A0)+	;VOLUME LABLE ATTRIBUTE
	BSR	XTRAP
	MOVE.B	3(A0),D0	;RETURN STATUS
	RTS
;------ BUILD A SECTOR IN MEMORY
BLDSEC	LEA	IDPAT(A6),A0	;HEADER PATERN
	MOVEQ	#21,D0
	BSR	MOVE
	MOVE.B	#$4E,D1		;GAP 2
	MOVEQ	#22,D0
	BSR	FILL
	LEA	DATPAT(A6),A0	;DATA FIELD
	MOVEQ	#16,D0
	BSR	MOVE
	MOVE	#$E5,D1		;DATA REC
	MOVE	#512,D0
	BSR	FILL
	MOVE.B	#$F7,(A1)+	;CRC WORD
	MOVE	#$4E,D1		;GAP 3
	MOVEQ	#GAP3,D0
	BSR	FILL
	RTS	
;------ FILL BUFFER
FILL	MOVE.B	D1,(A1)+
	SUBQ.L	#1,D0
	BNE	FILL
	RTS
;------ MOVE A BLOCK OF MEMORY
MOVE	MOVE.B	(A0)+,(A1)+
	SUBQ.L	#1,D0
	BNE	MOVE
	RTS
;------	GET A LINE FROM THE CONSOLE
GETLN	MOVEQ	#80,D0
	BRA.S	GET
;------ GET A CHARACTER FROM THE CONSOLE
GETC	MOVEQ	#1,D0		;GET ONE CHAR
GET	LEA	PARAM(A6),A0	;POINT TO PARAM BLK
	MOVE.W	#2,(A0)		;FUNCTION #2, READ
	CLR.W	2(A0)		;STATUS
	MOVE.W	#1,4(A0)	;CHAN = 1
	MOVE.W	D0,6(A0)	;COUNT
	LEA	GETBF(A6),A1	;POINT TO CHAR BUF
	MOVE.L	A1,10(A0)	;IS BUF POINTER
	BSR	XTRAP
	MOVE.W	8(A0),D0	;ACTUAL
	CLR.B	0(A1,D0.W)	;TERMINATE STRING WITH NULL
	MOVEQ	#0,D0
	MOVE.B	(A1),D0		;GET CHAR
	ANDI	#$7F,D0		;STRIP PARITY IF ANY
	RTS			;BRING IT BACK
;------ WRITE A LINE WITH CRLF
WRITELN	BSR	SPRINT		;THEN FALL INTO CRLF
;------ SEND CRLF TO CONSOLE
CRLF	LEA	CRXX(A6),A0	;FALL INTO SPRINT
;------ STRING PRINT ROUTINE -- NULL IS TERMINAL
SPRINT	MOVE.L	A0,D0		;COPY POINTER
SPR005	TST.B	(A0)+
	BNE	SPR005
	EXG	A0,D0
	SUB.L	A0,D0
	SUBQ.L	#1,D0		;FIX FOR ()+
SPR100	MOVE.W	#3,PARAM(A6)	;SET FOR WRITE
	CLR.L	PARAM+2(A6)	;CLEAR STATUS
	MOVE.W	#1,PARAM+4(A6)	;SEND TO CONSOLE
	MOVE.W	D0,PARAM+6(A6)	;BYTE COUNT
	MOVE.L	A0,PARAM+10(A6)	;BUF POINTER
;------ CALL FOR K-OS SERVICE
XTRAP	LEA	PARAM(A6),A0
	TRAP	#1
TRP2	TST.W	2(A0)		;WAIT FOR DONE
	BPL	TRP2
	RTS
***********************************************************
*        DATA STORAGE AREA
SIGNON	DC.B	'FORMAT --- COPYRIGHT 1986 HAWTHORNE TECHNOLOGY',13,10,10
	DC.B	'>>> WARNING: THIS PROGRAM WILL ERASE ALL DATA'
	DC.B	' ON THE DISK IN DRIVE A: <<<',13,10,10
	DC.B	'Insert disk into drive A:',13,10
	DC.B	'Press return to format disk, any other char to abort =>',0
DSKRMS	DC.B	'*** DISK ERROR DETECTED, FORMAT FAILED ***',0
VOLMSG	DC.B	'Enter volume name (return for none) =>',0
CRXX	DC.B	10,13,0
IDPAT	DC.B	0,0,0,0,0,0
	DC.B	0,0,0,0,0,0
	DC.B	$F5,$F5,$F5
	DC.B	$FE,0,0,0,2,$F7
DATPAT	DC.B	0,0,0,0,0,0
	DC.B	0,0,0,0,0,0
	DC.B	$F5,$F5,$F5,$FB
FATPAT	DC.B	$FD,$FF,$FF
	EVEN
;--------- DS AREA
PARAM	DS.W	30	;GENERAL PARAMETER BLOCK
COUNT	DS.L	1	;LOOP COUNTER
TRKCNT	DS.W	1	;TRACK COUNTER
GETBF	DS.B	80	;CONSOLE INPUT BUFFER
	DS.L	64
STACK	DS.L	4
	ORG	$ + 0FFH & 0FFFFFF00H
WRITBF	DS.B	4000H
READBF	EQU	$
	END
